"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SocksProxyAgent = exports.HttpsProxyAgent = void 0;
const net = __importStar(require("net"));
const tls = __importStar(require("tls"));
const assert_1 = __importDefault(require("assert"));
const dns_1 = __importDefault(require("dns"));
const agent_base_1 = require("agent-base");
const socks_1 = require("socks");
const parse_proxy_response_1 = __importDefault(require("./parse-proxy-response"));
const debug_1 = __importDefault(require("debug"));
const debug = (0, debug_1.default)('node-proxy-agent:agent');
function _normalizeProxyOptions(input) {
    let proxyOptions;
    if (typeof input === 'string')
        proxyOptions = new URL(input);
    else
        proxyOptions = input;
    if (!proxyOptions)
        throw new TypeError('A proxy server `host` and `port` must be specified!');
    return proxyOptions;
}
/**
 * The `HttpsProxyAgent` implements an HTTP Agent subclass that connects to
 * the specified "HTTP(s) proxy server" in order to proxy HTTPS requests.
 *
 * Outgoing HTTP requests are first tunneled through the proxy server using the
 * `CONNECT` HTTP request method to establish a connection to the proxy server,
 * and then the proxy server connects to the destination target and issues the
 * HTTP request from the proxy server.
 *
 * `https:` requests have their socket connection upgraded to TLS once
 * the connection to the proxy server has been established.
 *
 * @api public
 */
class HttpsProxyAgent extends agent_base_1.Agent {
    constructor(input, options) {
        var _a, _b, _c;
        // @ts-ignore
        const proxyOptions = Object.assign(_normalizeProxyOptions(input), options || {});
        super(proxyOptions);
        this.proxy = HttpsProxyAgent._parseHttpsProxy(proxyOptions);
        // If `true`, then connect to the proxy server over TLS.
        // Defaults to `false`.
        this.secureEndpoint = Boolean((_a = this.proxy.protocol) === null || _a === void 0 ? void 0 : _a.startsWith('https'));
        this.tlsSecureContext = proxyOptions.tls ? tls.createSecureContext(proxyOptions.tls) : undefined;
        this.ca = ((_b = proxyOptions.tls) === null || _b === void 0 ? void 0 : _b.ca) || proxyOptions.ca || undefined;
        this.timeout = (_c = proxyOptions.timeout) !== null && _c !== void 0 ? _c : null;
    }
    static _parseHttpsProxy(opts) {
        const proxy = opts;
        if (typeof proxy.port === 'string')
            proxy.port = parseInt(proxy.port, 10);
        // Use default port for proxy servers if not specified.
        else if (!proxy.port && proxy.host)
            proxy.port = proxy.secureEndpoint ? 443 : 80;
        // ALPN is supported by Node.js >= v5.
        // attempt to negotiate http/1.1 for proxy servers that support http/2
        if (proxy.secureEndpoint && !('ALPNProtocols' in proxy))
            proxy.ALPNProtocols = ['http 1.1'];
        // If both a `host` and `path` are specified then it's most likely
        // the result of a `url.parse()` call... we need to remove the
        // `path` portion so that `net.connect()` doesn't attempt to open
        // that as a Unix socket file.
        if (proxy.host && proxy.path) {
            delete proxy.path;
            delete proxy.pathname;
        }
        // Setup Proxy Auth
        if (!proxy.auth && (opts.username || opts.password))
            proxy.auth = `${opts.username || ''}:${opts.password || ''}`;
        return proxy;
    }
    /**
     * Called when the node-core HTTP client library is creating a
     * new HTTP request.
     *
     * @api protected
     */
    callback(req, opts) {
        return __awaiter(this, void 0, void 0, function* () {
            const { proxy, secureEndpoint } = this;
            // Create a socket connection to the proxy server.
            debug(`Creating \`${secureEndpoint ? 'tls' : 'net'}.Socket\`: %o`, proxy);
            const socket = secureEndpoint ? tls.connect(proxy) : net.connect(proxy);
            if (this.timeout != null && this.timeout >= 0) {
                socket.setTimeout(this.timeout);
                socket.on('timeout', () => {
                    debug('Socket timeout on connect');
                    socket.end();
                });
            }
            const headers = Object.assign({}, proxy.headers);
            const hostname = `${opts.host}:${opts.port}`;
            let payload = `CONNECT ${hostname} HTTP/1.1\r\n`;
            // Inject the `Proxy-Authorization` header if necessary.
            if (proxy.auth)
                headers['Proxy-Authorization'] = `Basic ${btoa(proxy.auth)}`;
            headers.Host = hostname;
            headers.Connection = 'close';
            for (const name of Object.keys(headers))
                payload += `${name}: ${headers[name]}\r\n`;
            const proxyResponsePromise = (0, parse_proxy_response_1.default)(socket);
            socket.write(`${payload}\r\n`);
            const { statusCode, buffered } = yield proxyResponsePromise;
            debug(`Response from proxy server: ${statusCode}`);
            if (statusCode === 200) {
                req.once('socket', (socket) => socket.resume());
                if (opts.secureEndpoint) {
                    // The proxy is connecting to a TLS server, so upgrade
                    // this socket connection to a TLS connection.
                    debug('Upgrading socket connection to TLS');
                    const tlsSocket = tls.connect(Object.assign(Object.assign({}, omit(opts, 'host', 'hostname', 'path', 'port')), { socket, servername: opts.servername || opts.host, secureContext: this.tlsSecureContext }));
                    return new Promise((resolve, reject) => {
                        const errCb = (err) => reject(err);
                        tlsSocket.once('error', errCb);
                        tlsSocket.once('secureConnect', () => {
                            resolve(tlsSocket);
                            tlsSocket.off('error', errCb);
                        });
                    });
                }
                return socket;
            }
            // Some other status code that's not 200... need to re-play the HTTP
            // header "data" events onto the socket once the HTTP machinery is
            // attached so that the node core `http` can parse and handle the
            // error status code.
            // Close the original socket, and a new "fake" socket is returned
            // instead, so that the proxy doesn't get the HTTP request
            // written to it (which may contain `Authorization` headers or other
            // sensitive data).
            //
            // See: https://hackerone.com/reports/541502
            socket.destroy();
            // Need to wait for the "socket" event to re-play the "data" events.
            req.once('socket', (s) => {
                debug('replaying proxy buffer for failed request');
                (0, assert_1.default)(s.listenerCount('data') > 0);
                // Replay the "buffered" Buffer onto the fake `socket`, since at
                // this point the HTTP module machinery has been hooked up for
                // the user.
                s.push(buffered);
                s.push(null);
            });
            return new net.Socket({ writable: false, readable: true });
        });
    }
}
exports.HttpsProxyAgent = HttpsProxyAgent;
/**
 * The `SocksProxyAgent` implements an HTTP Agent subclass that connects to
 * the specified "SOCKS proxy server" in order to proxy HTTPS requests.
 *
 * @api public
 */
class SocksProxyAgent extends agent_base_1.Agent {
    constructor(input, options) {
        var _a;
        // @ts-ignore
        const proxyOptions = Object.assign(_normalizeProxyOptions(input), options || {});
        super(proxyOptions);
        this.proxy = SocksProxyAgent._parseSocksProxy(proxyOptions);
        this.tlsConnectionOptions = proxyOptions.tls != null ? proxyOptions.tls : {};
        this.timeout = (_a = proxyOptions.timeout) !== null && _a !== void 0 ? _a : null;
    }
    static _parseSocksProxy(opts) {
        let port;
        let type = 5;
        if (!opts.hostname)
            throw new TypeError('You didn\'t specify "hostname" in options!');
        if (typeof opts.port === 'string')
            port = parseInt(opts.port, 10);
        else if (typeof opts.port === 'number')
            port = opts.port;
        /*
            From RFC 1928, Section 3: https://tools.ietf.org/html/rfc1928#section-3
            "The SOCKS service is conventionally located on TCP port 1080"
         */
        else
            port = 1080;
        if (!opts.type)
            opts.type = opts.protocol === 'socks5' ? 5 : 4;
        if (![4, 5].includes(opts.type))
            throw new TypeError(`"type" must be 4 or 5, got: ${String(opts.type)}`);
        const proxy = {
            host: opts.hostname,
            port,
            type
        };
        // Setup Proxy Auth
        let username;
        let password;
        if (opts.auth) {
            [username, password] = opts.auth.split(':');
        }
        else {
            username = opts.username;
            password = opts.password;
        }
        if (username) {
            Object.defineProperty(proxy, 'userId', {
                value: username,
                enumerable: false
            });
        }
        if (password) {
            Object.defineProperty(proxy, 'password', {
                value: password,
                enumerable: false
            });
        }
        return proxy;
    }
    /**
     * Initiates a SOCKS connection to the specified SOCKS proxy server,
     * which in turn connects to the specified remote host and port.
     *
     * @api protected
     */
    callback(req, opts) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const { proxy, timeout } = this;
            let { host, port, lookup: lookupCallback } = opts;
            if (!host)
                throw new Error('No `host` defined!');
            // Client-side DNS resolution for "4" and "5" socks proxy versions.
            host = yield new Promise((resolve, reject) => {
                // Use the request's custom lookup, if one was configured:
                const lookupFn = lookupCallback !== null && lookupCallback !== void 0 ? lookupCallback : dns_1.default.lookup;
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                try {
                    lookupFn(host, {}, (err, res) => err ? reject(err) : resolve(res));
                }
                catch (e) {
                    reject(e);
                }
            });
            const socksOpts = {
                proxy,
                destination: { host, port },
                command: 'connect',
                timeout: timeout !== null && timeout !== void 0 ? timeout : undefined
            };
            const cleanup = (tlsSocket) => {
                req.destroy();
                socket.destroy();
                if (tlsSocket)
                    tlsSocket.destroy();
            };
            debug('Creating socks proxy connection: %o', socksOpts);
            const { socket } = yield socks_1.SocksClient.createConnection(socksOpts);
            debug('Successfully created socks proxy connection');
            if (timeout !== null) {
                socket.setTimeout(timeout);
                socket.on('timeout', () => cleanup());
            }
            if (opts.secureEndpoint) {
                // The proxy is connecting to a TLS server, so upgrade
                // this socket connection to a TLS connection.
                debug('Upgrading socket connection to TLS');
                const servername = (_a = opts.servername) !== null && _a !== void 0 ? _a : opts.host;
                const tlsSocket = tls.connect(Object.assign(Object.assign(Object.assign({}, omit(opts, 'host', 'hostname', 'path', 'port')), { socket,
                    servername }), this.tlsConnectionOptions));
                tlsSocket.once('error', (error) => {
                    debug('socket TLS error', error.message);
                    cleanup(tlsSocket);
                });
                return tlsSocket;
            }
            return socket;
        });
    }
}
exports.SocksProxyAgent = SocksProxyAgent;
function omit(obj, ...keys) {
    const ret = {};
    let key;
    for (key in obj)
        if (!keys.includes(key))
            ret[key] = obj[key];
    return ret;
}
//# sourceMappingURL=index.js.map